/***
 * Copyright (C) 1996-98. by Miroslav Zagorac <zaga@tirnanog.cc.fer.hr>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>

#include "../data/objects_def.h"
#include "../lib/lib_debug.h"

#include "graphics.h"
#include "help.h"
#include "main.h"


static struct HELP_DATA editor_keys[] =
{
   {                    "", "Editor keys:"                   },
   {             "SHIFT c", "clear cave"                     },
   {             "SHIFT l", "load cave"                      },
   {             "SHIFT s", "save cave"                      },
   {             "SHIFT q", "exit editor"                    },
   {             "SHIFT u", "undo last action"               },
   {                    "", ""                               },
   {             "  ENTER", "enter/leave parameters section" },
   {       "     \032\033", "change active parameter"        },
   {       "     \030\031", "change parameter value"         },
   {                    "", ""                               },
   { "   \032\033\030\031", "move cursor"                    },
   {             "    INS", "insert/overwrite"               },
   {             "      h", "show this text"                 },
   {                   NULL, NULL                            }
};
static struct HELP_DATA game_keys[] =
{
   {   "", "Game keys:"                },
   {  "x", "left",                     },
   {  "c", "right",                    },
   {  "o", "up",                       },
   {  "k", "down",                     },
   {  "0", "fire",                     },
   {  "q", "quit a cave",              },
   {  "s", "reset game",               },
   {   "", ""                          },
   {  "p", "select number of players", },
   {  "e", "start level editor",       },
   {  "t", "show top ten score table", },
   {  "h", "show this text",           },
   { NULL, NULL                        }
};


#ifdef ANSI_FUNC

static void draw_border (short x, short y, short width, short height)
#else

static void draw_border (x, y, width, height)
   short x;
   short y;
   short width;
   short height;
#endif
{
   short i, j;

   for (j = 0; j < height; j++)
      for (i = 0; i < width; i++)
         if (!i || !j || (i == (width - 1)) || (j == (height - 1)))
            graph_putbox (x + i * CHAR_16x16, y + j * CHAR_16x16, CHAR_16x16, CHAR_16x16, wall_panel);
}


#ifdef ANSI_FUNC

void help_show (struct HELP_DATA help_data[], short *y, short width, short height)
#else
   
void help_show (help_data, y, width, height)
   struct HELP_DATA help_data[];
   short *y;
   short width;
   short height;
#endif
{
   short i, x_dist = 0, x = (80 * CHAR_8x8 - width) / 2, y_min = WINDOW_HEIGHT - height;

   debug (("{%s(0x%08lx, 0x%08lx <%d>, %d, %d)", __FUNCTION__, (long) help_data, (long) y, *y, width, height))

   if (*y >= WINDOW_HEIGHT)
      *y = WINDOW_HEIGHT;
   else
   {
      *y = (*y < y_min) ? y_min : *y;

      graph_fillbox (x + (2 * CHAR_8x8), *y + (2 * CHAR_8x8), width - 4 * CHAR_8x8, height - 4 * CHAR_8x8, BLACK);
      draw_border (x, *y, width / CHAR_16x16, height / CHAR_16x16);
      graph_printf (x + 3 * CHAR_8x8, *y + 3 * CHAR_8x8, RED, bd_font_8x8, help_data[0].description);

      for (i = 1; help_data[i].keys; i++)
      {
         short len = strlen (help_data[i].keys) * CHAR_8x8;

         x_dist = (x_dist < len) ? len : x_dist;
      }

      for (i = 1; help_data[i].keys; i++)
      {
         graph_printf (x + 3 * CHAR_8x8, *y + (4 + 1.25 * i) * CHAR_8x8, LIGHTGREEN, bd_font_8x8, help_data[i].keys);
         graph_printf (x + 4 * CHAR_8x8 + x_dist, *y + (4 + 1.25 * i) * CHAR_8x8, LIGHTGRAY, bd_font_8x8, help_data[i].description);
      }
   }

   debug_endfunc
}


#ifdef ANSI_FUNC

void show_editor_help (short direction)
#else
   
void show_editor_help (direction)
   short direction;
#endif
{
   static short y = WINDOW_HEIGHT;
   static float y_step = 12.0;

   debug (("{%s(%d)", __FUNCTION__, direction))

   if (direction > 0)
      y_step -= (y_step >= 1.35) ? 0.35 : 0.0;
   else
      y_step += (y_step <= 11.65) ? 0.35 : 0.0;

   y -= y_step * direction;

   help_show (editor_keys, &y, 44 * CHAR_8x8, 24 * CHAR_8x8);

   debug_endfunc
}


#ifdef ANSI_FUNC

void show_game_help (short direction)
#else
   
void show_game_help (direction)
   short direction;
#endif
{
   static short y = WINDOW_HEIGHT;
   static float y_step = 11.0;

   debug (("{%s(%d)", __FUNCTION__, direction))

   if (direction > 0)
      y_step -= (y_step >= 1.35) ? 0.35 : 0.0;
   else
      y_step += (y_step <= 10.65) ? 0.35 : 0.0;

   y -= y_step * direction;

   help_show (game_keys, &y, 32 * CHAR_8x8, 24 * CHAR_8x8);

   debug_endfunc
}
